"""Container to stack elements with spacing."""

from pynecone.components.libs.chakra import ChakraComponent
from pynecone.components.tags import Tag
from pynecone.var import Var


class Fade(ChakraComponent):
    """Display a square box."""

    tag = "Fade"


class ScaleFade(ChakraComponent):
    """Display a square box."""

    tag = "ScaleFade"


class Slide(ChakraComponent):
    """Display a square box."""

    tag = "Slide"


class SlideFade(ChakraComponent):
    """Display a square box."""

    tag = "SlideFade"
